Scriptname BlackSummer_MainScript extends activemagiceffect

BlackSummer_Quest Property Settings Auto
Actor Property PlayerRef Auto
ActorBase Property LvlFeralGhoulAggro1536 Auto
ActorBase Property LvlFeralGhoulGlowingOneAggro1024 Auto

Activator[] Property AshPiles Auto
Keyword Property BlackSummer_Converted Auto

Race Property HumanRace Auto
Race Property HumanRaceSubGraphData Auto
Race Property GhoulRace Auto
Race Property SuperMutantRace Auto
Race Property FeralGhoulRace Auto

Sound Property NPCFeralGhoulAmbushGetUpA Auto
Sound Property NPCFeralGhoulAmbushGetUpB Auto
Sound Property NPCFeralGhoulAmbushGetUpC Auto

MiscObject Property SynthDeathItem Auto

Actor CurrentActor
Actor feralToSpawn
string[] limbs
bool crippledHead = false
int crippleCount = 0
int deadCrippleCount = 0


;===== SETUP =====

Event OnEffectStart(Actor akTarget, Actor akCaster)
	CurrentActor = akTarget
	SetLimbs()
endEvent

Function SetLimbs()
	limbs = new string[23]
	limbs[0] = "Torso"
	limbs[1] = "Head1"
	limbs[2] = "Eye"
	limbs[3] = "LookAt"
	limbs[4] = "Fly Grab"
	limbs[5] = "LeftArm1"
	limbs[6] = "LeftArm2"
	limbs[7] = "RightArm1"
	limbs[8] = "RightArm2"
	limbs[9] = "LeftLeg1"
	limbs[10] = "LeftLeg2"
	limbs[11] = "LeftLeg3"
	limbs[12] = "RightLeg1"
	limbs[13] = "RightLeg2"
	limbs[14] = "RightLeg3"
	limbs[15] = "Brain"
	limbs[16] = "Weapon"
	limbs[17] = "COM"
	limbs[18] = "Pelvis"
	limbs[19] = "Camera"
	limbs[20] = "Offset Root"
	limbs[21] = "Left Foot"
	limbs[22] = "Right Foot"
endFunction


;===== CHECKS / VARIABLES =====

Bool Function IsEligible()
	if !currentActor.IsEssential()
		if IsHumanoid() || IsMutant() || IsFeral()
			return true
		endIf
	endIf
	return false
endFunction

Bool Function IsHumanoid()
	Race actorRace = currentActor.GetRace()
	if actorRace == HumanRace || actorRace == PlayerRef.GetRace() || actorRace == HumanRaceSubGraphData || actorRace == GhoulRace
		return true
	endIf
	return false
endFunction

bool Function IsMutant()
	if currentActor.GetRace() == SuperMutantRace
		return true
	endIf
	return false
endFunction

Bool Function IsFeral()
	if currentActor.GetRace() == FeralGhoulRace
		return true
	endIf
	return false
endFunction


;===== STATUS =====

Bool Function CanLiveAgain()
	If currentActor.GetItemCount(SynthDeathItem) > 0 || ShouldStayDead()
		return false
	elseif IsHumanoid() || IsMutant() || IsFeral()
		return true
	endIf
	return false
endFunction

Bool Function ShouldStayDead()  ;=== called from CanLiveAgain()
	If currentActor.IsDismembered("Head1") || currentActor.IsDismembered("Head2") || currentActor.IsDismembered("Brain")
		return true
	endIf
	If !Settings.HardMode
		if crippledHead || GetDismembermentCount() >= 4 || crippleCount >= 4 || deadCrippleCount >= 2
			return true
		endIf
	endIf
	If HasAshPile()
		return true
	endIf
	return false
endFunction

Bool Function HasAshPile()
	int index = 0
	while index < AshPiles.Length
		if currentActor.GetItemCount(ashPiles[index]) > 0
			return true
		endIf
		index += 1
	endWhile
	return false
endFunction

int Function GetDismembermentCount()
	int index = 0
	int dismemberCount = 0
	while index < 23
		if CurrentActor.IsDismembered(limbs[index])
			dismemberCount += 1
		endIf
		index += 1
	endWhile
	return dismemberCount
endFunction

Int Function GetChanceForRace()
	If IsMutant()
		return Settings.MutantChance
	elseIf IsFeral()
		return Settings.FeralChance
	else
		return Settings.HumanChance
	endIf
endFunction

Int Function GetTimeToWait()
 	int timeToWait = Utility.RandomInt(2, 6)
	if IsMutant()
		timeToWait = Utility.RandomInt(2, 10)
	elseIf IsFeral()
		timeToWait = Utility.RandomInt(2, 4)
	endIf
	return timeToWait
endFunction



;===== EVENTS =====

Event OnDeath(Actor akKiller)
	SetNPCKillChance(akKiller)
	if IsEligible() && CanLiveAgain()
		PrepFeral()
		int chanceToResurrect = Utility.RandomInt(0, 100)
		if chanceToResurrect <= GetChanceForRace()
			int WaitTime = GetTimeToWait()
			Utility.Wait(waitTime / 2)
			if ShouldStayDead()
				RemoveReplacementFeral()
			endIf
			Utility.Wait(waitTime / 2)
			ResurrectAsGhoul()
		else
			RemoveReplacementFeral()
		endIf
	endIf
endEvent

Function SetNPCKillChance(Actor akKiller)
	if IsEligible() && akKiller != PlayerRef && IsFeral()
		Int ChanceToDie = Utility.RandomInt(0, 100)
		Int NPCKillChance = Settings.NPCKillChance
		if ChanceToDie < NPCKillChance
			crippledHead = true
			currentActor.Dismember("Head1", abForceDismember = true)
			if ChanceToDie < NPCKillChance / 2
				currentActor.Dismember("Head1", abForceDismember = true)
			endIf
		endIf
	endIf
endFunction

Event OnCripple(ActorValue akActorValue, bool abCrippled)
	if IsEligible() && abCrippled
		string name = akActorValue.GetName()
		if name == "Head"
			crippledHead = true
		elseIf name != ""
			crippleCount += 1
		endIf
		if currentActor.IsDead()
			deadCrippleCount += 1
			If !Settings.HardMode
				RemoveReplacementFeral()
			endIf
		endIf
	endIF
EndEvent

Event OnPartialCripple(ActorValue akActorValue, bool abCrippled)
	if IsEligible() && abCrippled
		if akActorValue.GetName() == "Head"
			crippledHead = true
		endIf
	endIF
endEvent


;=====  RESURRECTION =====

Function PrepFeral()
	if !ShouldStayDead()
		SetReplacementFeral()
		feralToSpawn.EnableAI(true, false)
		MoveToActorPosition()
		feralToSpawn.PushActorAway(feralToSpawn, 0.1)
		feralToSpawn.SetAlpha(0)
		Utility.Wait(2)
		feralToSpawn.EnableAI(false, false)
	endIF
endFunction

Function ResurrectAsGhoul()
	if !ShouldStayDead()
		EnableFeral()
		PlayFeralSound()
		RemoveCurrentActor()
	else
		RemoveReplacementFeral()
	endIf
endFunction

Function SetReplacementFeral()
	if IsHumanoid() || (IsFeral() && currentActor.HasKeyword(BlackSummer_Converted))
		feralToSpawn= CurrentActor.PlaceAtMe(LvlFeralGhoulAggro1536, 1, abInitiallyDisabled = True, abDeleteWhenAble = false) As Actor
		feralToSpawn.AddKeyword(BlackSummer_Converted)
	elseIf IsMutant()
		feralToSpawn= CurrentActor.PlaceAtMe(LvlFeralGhoulGlowingOneAggro1024, 1, abInitiallyDisabled = True, abDeleteWhenAble = false) As Actor
		feralToSpawn.AddKeyword(BlackSummer_Converted)
	elseIf IsFeral()
		ActorBase feralBase = CurrentActor.GetActorBase()
		feralToSpawn= CurrentActor.PlaceAtMe(feralBase , 1, abInitiallyDisabled = True, abDeleteWhenAble = false) As Actor
	endIf
	if feralToSpawn != none
		Utility.Wait(0.2)
		feralToSpawn.RemoveAllItems()
	endIF
endFunction

Function MoveToActorPosition()
	float xPos = currentActor.GetPositionX()
	float yPos = currentActor.GetPositionY()
	float zPos = currentActor.GetPositionZ()
	feralToSpawn.MoveTo(currentActor, abMatchRotation = true)
	feralToSpawn.SetGhost(true)
	feralToSpawn.Enable(True)
	feralToSpawn.SetPosition(xPos, yPos, zPos)
endFunction

Function EnableFeral()
	currentActor.SetAlpha(0)
	feralToSpawn.EnableAI(true, true)
	feralToSpawn.SetAlpha(1)
	feralToSpawn.SetGhost(false)
	currentActor.RemoveAllItems(feralToSpawn)
	TransferDismembermentToNewFeral()
endFunction

Function PlayFeralSound()
	int random = Utility.RandomInt(0, 100) 
	if random < 33
		NPCFeralGhoulAmbushGetUpA.Play(currentActor)
	elseIf random < 66
		NPCFeralGhoulAmbushGetUpB.Play(currentActor)
	else
		NPCFeralGhoulAmbushGetUpC.Play(currentActor)
	endIF
endFunction

Function TransferDismembermentToNewFeral()
	int index = 0
	while index < 23
		if currentActor.IsDismembered(limbs[index])
			feralToSpawn.Dismember(limbs[index], abForceDismember = true)
		endIf
		index += 1
	endWhile
endFunction

Function RemoveCurrentActor()
	Utility.Wait(5)
	currentActor.Disable()
	Utility.Wait(1)
	currentActor.Delete()
endFunction

Function RemoveReplacementFeral()
	if feralToSpawn != none
		feralToSpawn.Disable()
		Utility.Wait(0.1)
		feralToSpawn.Delete()
	endIf
endFunction

